@@ -9,40 +9,127 @@ from .fields.utils import get_column_from_field, is_field_definition
 from .session import AsyncSession, get_session
 
 
+if TYPE_CHECKING:
+    from .metadata import ModelRegistry
+
+
 class _StateManager:
     """Unified state management for model instances."""
 
     def __init__(self):
         """Initialize empty state dictionary."""
-        self._state: dict[str, Any] = {}
+        self._state: dict[str, Any] = {
+            "dirty_fields": set(),
+            "deferred_fields": set(),
+            "loaded_deferred_fields": set(),
+            "is_from_db": False,
+            "bound_session": None,
+            "object_cache": {},
+            "needs_cascade_save": False,
+        }
 
-    def get(self, key: str, default=None):
-        """Get state value by key.
-
-        Args:
-            key: State key to retrieve
-            default: Default value if key not found
-
-        Returns:
-            State value or default
-        """
+    def _get(self, key: str, default):
+        """Internal method to get state value."""
         return self._state.get(key, default)
 
-    def set(self, key: str, value):
-        """Set state value by key.
-
-        Args:
-            key: State key to set
-            value: Value to store
-        """
+    def _set(self, key: str, value):
+        """Internal method to set state value."""
         self._state[key] = value
 
+    # Dirty fields management
+    def get_dirty_fields(self) -> set[str]:
+        """Get dirty fields set."""
+        return self._state["dirty_fields"]
+
+    def add_dirty_field(self, field_name: str):
+        """Add field to dirty fields."""
+        self._state["dirty_fields"].add(field_name)
+
+    def clear_dirty_fields(self):
+        """Clear all dirty fields."""
+        self._state["dirty_fields"].clear()
+
+    # Deferred fields management
+    def get_deferred_fields(self) -> set[str]:
+        """Get deferred fields set."""
+        return self._get("deferred_fields", set())
+
+    def set_deferred_fields(self, fields: set[str]):
+        """Set deferred fields."""
+        self._set("deferred_fields", fields)
+
+    def get_loaded_deferred_fields(self) -> set[str]:
+        """Get loaded deferred fields set."""
+        return self._get("loaded_deferred_fields", set())
+
+    def add_loaded_deferred_field(self, field_name: str):
+        """Mark deferred field as loaded."""
+        loaded_fields = self.get_loaded_deferred_fields()
+        loaded_fields.add(field_name)
+        self._set("loaded_deferred_fields", loaded_fields)
+
+    # Database state management
+    def is_from_database(self) -> bool:
+        """Check if instance was loaded from database."""
+        return bool(self._get("is_from_db", False))
+
+    def mark_from_database(self, from_db: bool = True):
+        """Mark instance as loaded from database."""
+        self._set("is_from_db", from_db)
+
+    # Session management
+    def get_bound_session(self):
+        """Get bound session."""
+        return self._get("bound_session", None)
+
+    def set_bound_session(self, session):
+        """Set bound session."""
+        self._set("bound_session", session)
+
+    # Object cache management
+    def get_object_cache(self) -> dict[str, Any]:
+        """Get object cache dictionary."""
+        return self._get("object_cache", {})
+
+    def update_object_cache(self, name: str, value: Any):
+        """Update object cache entry."""
+        cache = self.get_object_cache()
+        cache[name] = value
+        self._set("object_cache", cache)
+
+    def clear_cache_entry(self, name: str):
+        """Clear specific cache entry."""
+        cache = self.get_object_cache()
+        cache.pop(name, None)
+        self._set("object_cache", cache)
+
+    # Cascade relationships management
+    def get_cascade_relationships(self) -> dict[str, Any]:
+        """Get cascade relationships dictionary."""
+        return self._get("cascade_relationships", {})
+
+    def set_cascade_relationship(self, field_name: str, value: Any):
+        """Set cascade relationship value."""
+        cascade_relationships = self.get_cascade_relationships()
+        cascade_relationships[field_name] = value
+        self._set("cascade_relationships", cascade_relationships)
+        self._set("needs_cascade_save", True)
+
+    def needs_cascade_save(self) -> bool:
+        """Check if cascade save is needed."""
+        return bool(self._get("needs_cascade_save", False))
+
+    def clear_cascade_save_flag(self):
+        """Clear cascade save flag."""
+        self._set("needs_cascade_save", False)
+
 
 class BaseMixin:
     """Base mixin with common functionality and state management."""
 
     if TYPE_CHECKING:
         __table__: ClassVar[Table]
+        __registry__: ClassVar["ModelRegistry"]
 
     def __init__(self):
         """Initialize state manager if not already present."""
@@ -78,7 +165,7 @@ class SessionMixin(BaseMixin):
         Returns:
             AsyncSession instance for database operations
         """
-        bound_session = self._state_manager.get("bound_session")
+        bound_session = self._state_manager.get_bound_session()
         if isinstance(bound_session, str):
             return get_session(bound_session)
         return bound_session or get_session()
@@ -92,7 +179,7 @@ class SessionMixin(BaseMixin):
         Returns:
             Self with bound session for method chaining
         """
-        self._state_manager.set("bound_session", db_or_session)
+        self._state_manager.set_bound_session(db_or_session)
         return self
 
 
@@ -128,6 +215,9 @@ class PrimaryKeyMixin(SessionMixin):
             True if all primary key fields have non-None values
         """
         pk_values = self._get_primary_key_values()
+        # Handle case where model has no primary key fields
+        if not pk_values:
+            return False
         return all(value is not None for value in pk_values.values())
 
     def _build_pk_conditions(self) -> list:
@@ -203,22 +293,13 @@ class ValidationMixin(PrimaryKeyMixin):
 class DeferredLoadingMixin(ValidationMixin):
     """Deferred loading functionality - Layer 4."""
 
-    def __init__(self):
-        """Initialize deferred loading state."""
-        super().__init__()
-        self._state_manager.set("deferred_fields", set())
-        self._state_manager.set("loaded_deferred_fields", set())
-        self._state_manager.set("is_from_db", False)
-
     @property
     def _deferred_fields(self) -> set[str]:
-        result = self._state_manager.get("deferred_fields", set())
-        return result if isinstance(result, set) else set()
+        return self._state_manager.get_deferred_fields()
 
     @property
     def _loaded_deferred_fields(self) -> set[str]:
-        result = self._state_manager.get("loaded_deferred_fields", set())
-        return result if isinstance(result, set) else set()
+        return self._state_manager.get_loaded_deferred_fields()
 
     def get_deferred_fields(self) -> set[str]:
         """Get all deferred fields.
@@ -226,7 +307,7 @@ class DeferredLoadingMixin(ValidationMixin):
         Returns:
             Set of field names that are deferred
         """
-        return self._deferred_fields.copy()
+        return self._state_manager.get_deferred_fields().copy()
 
     def get_loaded_deferred_fields(self) -> set[str]:
         """Get loaded deferred fields.
@@ -234,7 +315,7 @@ class DeferredLoadingMixin(ValidationMixin):
         Returns:
             Set of deferred field names that have been loaded
         """
-        return self._loaded_deferred_fields.copy()
+        return self._state_manager.get_loaded_deferred_fields().copy()
 
     def is_field_deferred(self, field_name: str) -> bool:
         """Check if field is deferred.
@@ -266,8 +347,7 @@ class DeferredLoadingMixin(ValidationMixin):
         Returns:
             True if instance was loaded from database
         """
-        result = self._state_manager.get("is_from_db", False)
-        return bool(result)
+        return self._state_manager.is_from_database()
 
     async def load_deferred_field(self, field_name: str) -> None:
         """Load a single deferred field.
@@ -312,11 +392,11 @@ class DeferredLoadingMixin(ValidationMixin):
         row = result.first()
 
         if row:
-            loaded_fields = self._state_manager.get("loaded_deferred_fields", set())
-            if isinstance(loaded_fields, set):
-                for i, field in enumerate(valid_fields):
-                    setattr(self, field, row[i])
-                    loaded_fields.add(field)
+            for i, field in enumerate(valid_fields):
+                value = row[i]
+                # Store actual value in proxy_cache (replaces proxy object)
+                self._state_manager.update_object_cache(field, value)
+                self._state_manager.add_loaded_deferred_field(field)
 
 
 class DataConversionMixin(DeferredLoadingMixin):
@@ -380,19 +460,7 @@ class DataConversionMixin(DeferredLoadingMixin):
         all_fields = set(cls._get_field_names())
         filtered_data = {k: v for k, v in data.items() if k in all_fields}
 
-        table = cls.get_table()
-        for col in table.columns:  # noqa
-            if col.name not in filtered_data:
-                field_attr = getattr(cls, col.name, None)
-                if field_attr is not None and hasattr(field_attr, "get_default_factory"):
-                    factory = field_attr.get_default_factory()
-                    if factory and callable(factory):
-                        filtered_data[col.name] = factory()
-                        continue
-
-                if col.default is not None:
-                    if hasattr(col.default, "is_scalar") and col.default.is_scalar:
-                        filtered_data[col.name] = getattr(col.default, "arg", None)
+        # Default values will be handled by __init__ method
 
         init_data = {}
         non_init_data = {}
@@ -406,114 +474,127 @@ class DataConversionMixin(DeferredLoadingMixin):
                 else:
                     non_init_data[field_name] = value
             else:
-                init_data[field_name] = value
+                if field_name == "id" and hasattr(field_attr, "column") and hasattr(field_attr.column, "autoincrement"):  # type: ignore[reportOptionalSubscript]
+                    non_init_data[field_name] = value
+                else:
+                    init_data[field_name] = value
 
         instance = cls(**init_data)  # noqa
 
         for field_name, value in non_init_data.items():
+            if value is None:
+                default_value = instance._get_field_default_value(field_name)
+                if default_value is not None:
+                    value = default_value
             setattr(instance, field_name, value)
 
         # Clear dirty fields since this is initial creation from dict
-        dirty_fields = instance._state_manager.get("dirty_fields", set())
-        if isinstance(dirty_fields, set):
-            dirty_fields.clear()
+        instance._state_manager.clear_dirty_fields()
 
         if validate:
             instance.validate_all_fields()
 
         return instance
 
+    def _get_field_default_value(self, field_name: str):
+        """Get default value for a field.
+
+        Args:
+            field_name: Name of the field
+
+        Returns:
+            Default value or None if no default
+        """
+        field_attr = getattr(self.__class__, field_name, None)
+        if field_attr is None:
+            return None
+
+        if hasattr(field_attr, "get_default_factory"):
+            factory = field_attr.get_default_factory()
+            if factory and callable(factory):
+                return factory()
+
+        if hasattr(field_attr, "default") and field_attr.default is not None:
+            default_value = field_attr.default
+            if callable(default_value):
+                return default_value()
+            else:
+                return default_value
+
+        return None
+
+    def _apply_default_values(self, kwargs: dict):
+        """Apply default values for fields not provided in kwargs.
+
+        Args:
+            kwargs: Dictionary of provided field values (will be modified)
+        """
+        for field_name in self._get_field_names():
+            if field_name not in kwargs or kwargs[field_name] is None:
+                default_value = self._get_field_default_value(field_name)
+                if default_value is not None:
+                    kwargs[field_name] = default_value
+
 
 class FieldCacheMixin(DataConversionMixin):
     """Field caching and attribute access optimization - Layer 6."""
 
     @classmethod
     def _get_field_cache(cls):
-        """Auto-initialize and cache field information.
+        """Get field cache from metadata system.
 
         Returns:
             Dictionary containing categorized field information
         """
-        cache_attr = "_cached_field_info"
-        if not hasattr(cls, cache_attr):
-            setattr(cls, cache_attr, cls._build_field_cache())
-        return getattr(cls, cache_attr)
+        return getattr(
+            cls, "_field_cache", {"deferred_fields": set(), "relationship_fields": set(), "regular_fields": set()}
+        )
 
-    @classmethod
-    def _build_field_cache(cls):
-        """Build field cache with error handling.
-
-        Returns:
-            Dictionary with field categories: deferred_fields, relationship_fields, regular_fields
-        """
-        cache = {"deferred_fields": set(), "relationship_fields": set(), "regular_fields": set()}
-
-        try:
-            if hasattr(cls, "__table__"):
-                table = getattr(cls, "__table__", None)
-                if table is not None:
-                    for col_name in table.columns.keys():
-                        cls._categorize_field(col_name, cache)
-
-            if hasattr(cls, "_relationships"):
-                relationships = getattr(cls, "_relationships", {})
-                cache["relationship_fields"].update(relationships.keys())
-        except Exception:  # noqa
-            pass
-
-        return cache
-
-    @classmethod
-    def _categorize_field(cls, field_name, cache):
-        """Categorize a single field into cache.
+    def _update_cache(self, name: str, value: Any):
+        """Update object cache entry.
 
         Args:
-            field_name: Name of the field to categorize
-            cache: Cache dictionary to update
+            name: Field name
+            value: Value to cache
         """
-        try:
-            attr = getattr(cls, field_name, None)
-            if attr is not None and is_field_definition(attr):
-                # Check if this is a relationship field
-                if hasattr(attr, "_is_relationship") and attr._is_relationship:  # noqa
-                    cache["relationship_fields"].add(field_name)
-                    return
+        self._state_manager.update_object_cache(name, value)
 
-                # Handle database field
-                column = get_column_from_field(attr)
-                if column is not None and hasattr(column, "info") and column.info is not None:
-                    performance_params = column.info.get("_performance", {})
-                    if performance_params.get("deferred", False):
-                        cache["deferred_fields"].add(field_name)
-                    else:
-                        cache["regular_fields"].add(field_name)
-                else:
-                    cache["regular_fields"].add(field_name)
-        except (AttributeError, TypeError):
-            cache["regular_fields"].add(field_name)
+    def _clear_cache_entry(self, name: str):
+        """Clear specific cache entry.
 
-    @classmethod
-    def _invalidate_field_cache(cls):
-        """Manually invalidate field cache.
-
-        Use this when field definitions change at runtime.
+        Args:
+            name: Field name to clear
         """
-        cache_attr = "_cached_field_info"
-        if hasattr(cls, cache_attr):
-            delattr(cls, cache_attr)
+        self._state_manager.clear_cache_entry(name)
+
+    def _get_relationship_fields(self) -> set[str]:
+        """Get relationship field names from model metadata."""
+        relationships = getattr(self.__class__, "_relationships", {})
+        return set(relationships.keys())
+
+    def __setattr__(self, name: str, value):
+        """Override setattr to handle relationship field assignments."""
+        if hasattr(self, "_get_relationship_fields") and name in self._get_relationship_fields():
+            self._handle_relationship_assignment(name, value)
+        else:
+            super().__setattr__(name, value)
+
+    def _handle_relationship_assignment(self, field_name: str, value):
+        """Handle assignment of relationship objects for cascade save."""
+        if not hasattr(self, "_state_manager"):
+            return
+
+        relationship_value = value if isinstance(value, list) else [value]
+        self._state_manager.set_cascade_relationship(field_name, relationship_value)
 
     def __getattribute__(self, name: str):
-        """Optimized attribute access using automatic field cache.
-
-        Provides intelligent attribute access with proxy objects for
-        deferred and relationship fields. Skips optimization for
-        special attributes and methods to avoid recursion.
+        """Optimized attribute access using unified proxy cache.
 
         Args:
             name: Attribute name to access
 
         Returns:
-            Attribute value or proxy object
+            Attribute value, cached data, or proxy object
         """
         if name.startswith("_") or name in (
             "get_table",
@@ -528,7 +609,6 @@ class FieldCacheMixin(DataConversionMixin):
             "is_field_deferred",
             "is_field_loaded",
             "get_deferred_fields",
-            "_get_field_cache",
             "get_session",
             "validate_field",
             "load_deferred_field",
@@ -536,40 +616,38 @@ class FieldCacheMixin(DataConversionMixin):
         ):
             return super().__getattribute__(name)
 
-        model_class = super().__getattribute__("__class__")
-        field_cache = model_class._get_field_cache()  # noqa
+        # Check object cache first
+        object_cache = self._state_manager.get_object_cache()
+        if name in object_cache:
+            return object_cache[name]  # May be actual value, prefetch data, or proxy object
 
+        model_class = super().__getattribute__("__class__")
+        field_cache = model_class._get_field_cache()
+
+        # Handle deferred fields
         deferred_fields = field_cache.get("deferred_fields", set())
         if isinstance(deferred_fields, set) and name in deferred_fields:
             if (
-                hasattr(self, "_state_manager")
-                and self._state_manager.get("is_from_db", False)
-                and name in self._deferred_fields
+                name in self._deferred_fields
                 and not self.is_field_loaded(name)
+                and self._state_manager.is_from_database()
             ):
-                proxy_cache = self._state_manager.get("proxy_cache", {})
-                if isinstance(proxy_cache, dict) and name not in proxy_cache:
-                    proxy_cache[name] = DeferredFieldProxy(self, name)
-                    self._state_manager.set("proxy_cache", proxy_cache)
-                if isinstance(proxy_cache, dict):
-                    return proxy_cache[name]
+                # Create and cache deferred field proxy
+                proxy = DeferredFieldProxy(self, name)
+                self._update_cache(name, proxy)
+                return proxy
 
+        # Handle relationship fields
         relationship_fields = field_cache.get("relationship_fields", set())
         if isinstance(relationship_fields, set) and name in relationship_fields:
-            cache_name = f"_{name}_cache"
-            try:
-                if hasattr(self, cache_name):
-                    cached_value = super().__getattribute__(cache_name)
-                    if cached_value is not None:
-                        return cached_value
-            except AttributeError:
-                pass
+            # Check cascade_relationships (manually assigned values)
+            cascade_relationships = self._state_manager.get_cascade_relationships()
+            if name in cascade_relationships:
+                return cascade_relationships[name]
 
-            proxy_cache = self._state_manager.get("proxy_cache", {})
-            if isinstance(proxy_cache, dict) and name not in proxy_cache:
-                proxy_cache[name] = RelationFieldProxy(self, name)
-                self._state_manager.set("proxy_cache", proxy_cache)
-            if isinstance(proxy_cache, dict):
-                return proxy_cache[name]
+            # Create and cache relationship proxy
+            proxy = RelationFieldProxy(self, name)
+            self._update_cache(name, proxy)
+            return proxy
 
         return super().__getattribute__(name)
